# -*- coding: utf-8 -*-
# Time       : 2022/2/15 17:40
# Author     : QIN2DIM
# Github     : https://github.com/QIN2DIM
# Description: 🚀 Yo Challenger!
from fire import Fire

from services.scaffold import Scaffold

if __name__ == "__main__":
    Fire(Scaffold)
